@@ -4,11 +4,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452573;
-var summary = 'Do not assert with JIT: JSVAL_IS_VOID(boxed) || JSVAL_IS_BOOLEAN(boxed)';
+var BUGNUMBER = 387951;
+var summary = 'Do not assert: cg->stackDepth >= 0';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
+
 //-----------------------------------------------------------------------------
 test();
 //-----------------------------------------------------------------------------
@@ -18,12 +19,8 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  jit(true);
-
-  for(var j=0;j<5;++j) typeof void /x/;
-
-  jit(false);
+ 
+  switch(delete[null?0:{}]){default:}
 
   reportCompare(expect, actual, summary);
 
